(*
 * Copyright 2020 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *)

[inherit('sqlite')]
module psqlite;

type
   byte = [byte] -128..127;
   word = [word] -32768..32767;
   pstr = varying[32000] of char;
   longpstr(n : integer)  = record
                               length : integer;
                               body : packed array[1..n] of char;
                            end;

function stdstr(s : longpstr) : pstr;

begin
   stdstr := substr(s.body, 1, s.length);
end;

function psqlite_open(db : pstr) : sqlite_ptr;

var
   xdb : c_str_t;
   con : sqlite_ptr;
   stat : integer;

begin
   crtl_init;
   xdb := malloc_c_str(db);
   stat := sqlite3_open(xdb, con);
   free_c_str(xdb);
   if stat = SQLITE_OK then
      psqlite_open := con
   else
      psqlite_open := 0;
end;

function psqlite_prepare(con : sqlite_ptr; sqlstr : pstr) : sqlite_stmt_ptr;

var
   xsqlstr : c_str_t;
   stmt : sqlite_stmt_ptr;
   stat : integer;

begin
   xsqlstr := malloc_c_str(sqlstr);
   stat := sqlite3_prepare(con, xsqlstr, length(sqlstr), stmt);
   free_c_str(xsqlstr);
   if stat = SQLITE_OK then
      psqlite_prepare := stmt
   else
      psqlite_prepare := 0;
end;

function psqlite_bind_int(stmt : sqlite_stmt_ptr; ix : integer; ival : integer) : boolean;

begin
   psqlite_bind_int := sqlite3_bind_int(stmt, ix, ival) = SQLITE_OK;
end;

function psqlite_bind_double(stmt : sqlite_stmt_ptr; ix : integer; xval : double) : boolean;

begin
   psqlite_bind_double := sqlite3_bind_double(stmt, ix, xval) = SQLITE_OK;
end;
                                                        
function psqlite_bind_text(stmt : sqlite_stmt_ptr; ix : integer; sval : pstr) : boolean;

var
   xsval : c_str_t;

begin
   xsval := malloc_c_str(sval);
   psqlite_bind_text := sqlite3_bind_text(stmt, ix, xsval, length(sval)) = SQLITE_OK;
   free_c_str(xsval);
end;

function psqlite_bind_null(stmt : sqlite_stmt_ptr; ix : integer) : boolean;

begin
   psqlite_bind_null := sqlite3_bind_null(stmt, ix) = SQLITE_OK;
end;

function psqlite_step_nonquery(stmt : sqlite_stmt_ptr) : boolean;

begin
    psqlite_step_nonquery := (sqlite3_step(stmt) = SQLITE_DONE);
end;

function psqlite_step_query(stmt : sqlite_stmt_ptr) : boolean;

begin
    psqlite_step_query := (sqlite3_step(stmt) = SQLITE_ROW);
end;

function psqlite_column_int(stmt : sqlite_stmt_ptr; ix : integer) : integer;

begin
   psqlite_column_int := sqlite3_column_int(stmt, ix);
end;

function psqlite_column_double(stmt : sqlite_stmt_ptr; ix : integer) : double;

begin
   psqlite_column_double := sqlite3_column_double(stmt, ix);
end;

function psqlite_column_text(stmt : sqlite_stmt_ptr; ix : integer) : pstr;

var
   tmp : c_str_t;
   res : pstr;

begin
   tmp := sqlite3_column_text(stmt, ix);
   res := pas_str(tmp);
   free_c_str(tmp);
   psqlite_column_text := res;
end;

procedure psqlite_finalize(stmt : sqlite_stmt_ptr);

begin
   sqlite3_finalize(stmt);
end;

procedure psqlite_close(con : sqlite_ptr);

begin
   sqlite3_close(con);
end;

function psqlite_errmsg(con : sqlite_ptr) : pstr;

begin
   psqlite_errmsg := pas_str(sqlite3_errmsg(con));
end;

end.
